@@ -41,16 +41,267 @@ struct info_t {
 struct compiled_format_t {
     PyObject_HEAD
     struct info_t *info_p;
+    PyObject *format_p;
 };
 
 struct compiled_format_dict_t {
     PyObject_HEAD
     struct info_t *info_p;
+    PyObject *format_p;
     PyObject *names_p;
 };
 
+static const char* pickle_version_key = "_pickle_version";
+static int pickle_version = 1;
+
+static PyObject *compiled_format_new(PyTypeObject *type_p,
+                                     PyObject *args_p,
+                                     PyObject *kwargs_p);
+
+static int compiled_format_init(struct compiled_format_t *self_p,
+                                PyObject *args_p,
+                                PyObject *kwargs_p);
+
+static int compiled_format_init_inner(struct compiled_format_t *self_p,
+                                      PyObject *format_p);
+
+static void compiled_format_dealloc(struct compiled_format_t *self_p);
+
+static PyObject *m_compiled_format_pack(struct compiled_format_t *self_p,
+                                        PyObject *args_p);
+
+static PyObject *m_compiled_format_unpack(struct compiled_format_t *self_p,
+                                          PyObject *args_p);
+
+static PyObject *m_compiled_format_pack_into(struct compiled_format_t *self_p,
+                                             PyObject *args_p,
+                                             PyObject *kwargs_p);
+
+static PyObject *m_compiled_format_unpack_from(struct compiled_format_t *self_p,
+                                               PyObject *args_p,
+                                               PyObject *kwargs_p);
+
+static PyObject *m_compiled_format_calcsize(struct compiled_format_t *self_p);
+
+static PyObject *m_compiled_format_copy(struct compiled_format_t *self_p);
+
+static PyObject *m_compiled_format_deepcopy(struct compiled_format_t *self_p,
+                                            PyObject *args_p);
+
+static PyObject *m_compiled_format_getstate(struct compiled_format_t *self_p,
+                                            PyObject *args_p);
+
+static PyObject *m_compiled_format_setstate(struct compiled_format_t *self_p,
+                                            PyObject *args_p);
+
+static PyObject *compiled_format_dict_new(PyTypeObject *type_p,
+                                          PyObject *args_p,
+                                          PyObject *kwargs_p);
+
+static int compiled_format_dict_init(struct compiled_format_dict_t *self_p,
+                                     PyObject *args_p,
+                                     PyObject *kwargs_p);
+
+static int compiled_format_dict_init_inner(struct compiled_format_dict_t *self_p,
+                                           PyObject *format_p,
+                                           PyObject *names_p);
+
+static void compiled_format_dict_dealloc(struct compiled_format_dict_t *self_p);
+
+static PyObject *m_compiled_format_dict_pack(struct compiled_format_dict_t *self_p,
+                                             PyObject *data_p);
+
+static PyObject *m_compiled_format_dict_unpack(
+    struct compiled_format_dict_t *self_p,
+    PyObject *data_p);
+
+static PyObject *m_compiled_format_dict_pack_into(
+    struct compiled_format_dict_t *self_p,
+    PyObject *args_p,
+    PyObject *kwargs_p);
+
+static PyObject *m_compiled_format_dict_unpack_from(
+    struct compiled_format_dict_t *self_p,
+    PyObject *args_p,
+    PyObject *kwargs_p);
+
+static PyObject *m_compiled_format_dict_calcsize(
+    struct compiled_format_dict_t *self_p);
+
+static PyObject *m_compiled_format_dict_copy(
+    struct compiled_format_dict_t *self_p);
+
+static PyObject *m_compiled_format_dict_deepcopy(
+    struct compiled_format_dict_t *self_p,
+    PyObject *args_p);
+
+static PyObject *m_compiled_format_dict_getstate(struct compiled_format_dict_t *self_p,
+                                                 PyObject *args_p);
+
+static PyObject *m_compiled_format_dict_setstate(struct compiled_format_dict_t *self_p,
+                                                 PyObject *args_p);
+
+PyDoc_STRVAR(pack___doc__,
+             "pack(fmt, *args)\n"
+             "--\n"
+             "\n");
+
+PyDoc_STRVAR(unpack___doc__,
+             "unpack(fmt, data)\n"
+             "--\n"
+             "\n");
+
+PyDoc_STRVAR(pack_into___doc__,
+             "pack_into(fmt, buf, offset, *args, **kwargs)\n"
+             "--\n"
+             "\n");
+
+PyDoc_STRVAR(unpack_from___doc__,
+             "unpack_from(fmt, data, offset=0)\n"
+             "--\n"
+             "\n");
+
+PyDoc_STRVAR(calcsize___doc__,
+             "calcsize(fmt)\n"
+             "--\n"
+             "\n");
+
 static PyObject *py_zero_p = NULL;
 
+static struct PyMethodDef compiled_format_methods[] = {
+    {
+        "pack",
+        (PyCFunction)m_compiled_format_pack,
+        METH_VARARGS,
+        pack___doc__
+    },
+    {
+        "unpack",
+        (PyCFunction)m_compiled_format_unpack,
+        METH_VARARGS,
+        unpack___doc__
+    },
+    {
+        "pack_into",
+        (PyCFunction)m_compiled_format_pack_into,
+        METH_VARARGS | METH_KEYWORDS,
+        pack_into___doc__
+    },
+    {
+        "unpack_from",
+        (PyCFunction)m_compiled_format_unpack_from,
+        METH_VARARGS | METH_KEYWORDS,
+        unpack_from___doc__
+    },
+    {
+        "calcsize",
+        (PyCFunction)m_compiled_format_calcsize,
+        METH_NOARGS,
+        calcsize___doc__
+    },
+    {
+        "__copy__",
+        (PyCFunction)m_compiled_format_copy,
+        METH_NOARGS
+    },
+    {
+        "__deepcopy__",
+        (PyCFunction)m_compiled_format_deepcopy,
+        METH_VARARGS
+    },
+    {
+        "__getstate__",
+        (PyCFunction)m_compiled_format_getstate,
+        METH_NOARGS
+    },
+    {
+        "__setstate__",
+        (PyCFunction)m_compiled_format_setstate,
+        METH_O
+    },
+    { NULL }
+};
+
+static PyTypeObject compiled_format_type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    .tp_name = "bitstruct.c.CompiledFormat",
+    .tp_doc = NULL,
+    .tp_basicsize = sizeof(struct compiled_format_t),
+    .tp_itemsize = 0,
+    .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+    .tp_new = compiled_format_new,
+    .tp_init = (initproc)compiled_format_init,
+    .tp_dealloc = (destructor)compiled_format_dealloc,
+    .tp_methods = compiled_format_methods,
+};
+
+static struct PyMethodDef compiled_format_dict_methods[] = {
+    {
+        "pack",
+        (PyCFunction)m_compiled_format_dict_pack,
+        METH_O,
+        pack___doc__
+    },
+    {
+        "unpack",
+        (PyCFunction)m_compiled_format_dict_unpack,
+        METH_O,
+        unpack___doc__
+    },
+    {
+        "pack_into",
+        (PyCFunction)m_compiled_format_dict_pack_into,
+        METH_VARARGS | METH_KEYWORDS,
+        pack_into___doc__
+    },
+    {
+        "unpack_from",
+        (PyCFunction)m_compiled_format_dict_unpack_from,
+        METH_VARARGS | METH_KEYWORDS,
+        unpack_from___doc__
+    },
+    {
+        "calcsize",
+        (PyCFunction)m_compiled_format_dict_calcsize,
+        METH_NOARGS,
+        calcsize___doc__
+    },
+    {
+        "__copy__",
+        (PyCFunction)m_compiled_format_dict_copy,
+        METH_NOARGS
+    },
+    {
+        "__deepcopy__",
+        (PyCFunction)m_compiled_format_dict_deepcopy,
+        METH_VARARGS
+    },
+    {
+        "__getstate__",
+        (PyCFunction)m_compiled_format_dict_getstate,
+        METH_NOARGS
+    },
+    {
+        "__setstate__",
+        (PyCFunction)m_compiled_format_dict_setstate,
+        METH_O
+    },
+    { NULL }
+};
+
+static PyTypeObject compiled_format_dict_type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    .tp_name = "bitstruct.c.CompiledFormatDict",
+    .tp_doc = NULL,
+    .tp_basicsize = sizeof(struct compiled_format_dict_t),
+    .tp_itemsize = 0,
+    .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+    .tp_new = compiled_format_dict_new,
+    .tp_init = (initproc)compiled_format_dict_init,
+    .tp_dealloc = (destructor)compiled_format_dict_dealloc,
+    .tp_methods = compiled_format_dict_methods,
+};
+
 static bool is_names_list(PyObject *names_p)
 {
     if (!PyList_Check(names_p)) {
@@ -720,11 +971,6 @@ static PyObject *pack(struct info_t *info_p,
     return (pack_finalize(packed_p));
 }
 
-PyDoc_STRVAR(pack___doc__,
-             "pack(fmt, *args)\n"
-             "--\n"
-             "\n");
-
 static PyObject *m_pack(PyObject *module_p, PyObject *args_p)
 {
     Py_ssize_t number_of_args;
@@ -802,11 +1048,6 @@ static PyObject *unpack(struct info_t *info_p, PyObject *data_p, long offset)
     return (unpacked_p);
 }
 
-PyDoc_STRVAR(unpack___doc__,
-             "unpack(fmt, data)\n"
-             "--\n"
-             "\n");
-
 static PyObject *m_unpack(PyObject *module_p, PyObject *args_p)
 {
     PyObject *format_p;
@@ -910,9 +1151,7 @@ static PyObject *pack_into_finalize(struct bitstream_writer_bounds_t *bounds_p)
         return (NULL);
     }
 
-    Py_INCREF(Py_None);
-
-    return (Py_None);
+    Py_RETURN_NONE;
 }
 
 static PyObject *pack_into(struct info_t *info_p,
@@ -943,11 +1182,6 @@ static PyObject *pack_into(struct info_t *info_p,
     return (pack_into_finalize(&bounds));
 }
 
-PyDoc_STRVAR(pack_into___doc__,
-             "pack_into(fmt, buf, offset, *args, **kwargs)\n"
-             "--\n"
-             "\n");
-
 static PyObject *m_pack_into(PyObject *module_p,
                              PyObject *args_p,
                              PyObject *kwargs_p)
@@ -1002,11 +1236,6 @@ static PyObject *unpack_from(struct info_t *info_p,
     return (unpack(info_p, data_p, offset));
 }
 
-PyDoc_STRVAR(unpack_from___doc__,
-             "unpack_from(fmt, data, offset=0)\n"
-             "--\n"
-             "\n");
-
 static PyObject *m_unpack_from(PyObject *module_p,
                                PyObject *args_p,
                                PyObject *kwargs_p)
@@ -1190,6 +1419,7 @@ static PyObject *unpack_dict(struct info_t *info_p,
             PyDict_SetItem(unpacked_p,
                            PyList_GET_ITEM(names_p, produced_args),
                            value_p);
+            Py_DECREF(value_p);
             produced_args++;
         }
     }
@@ -1392,11 +1622,6 @@ static PyObject *calcsize(struct info_t *info_p)
     return (PyLong_FromLong(info_p->number_of_bits));
 }
 
-PyDoc_STRVAR(calcsize___doc__,
-             "calcsize(fmt)\n"
-             "--\n"
-             "\n");
-
 static PyObject *m_calcsize(PyObject *module_p, PyObject *format_p)
 {
     PyObject *size_p;
@@ -1528,28 +1753,69 @@ static PyObject *m_byteswap(PyObject *module_p, PyObject *args_p)
     return (NULL);
 }
 
-static PyObject *compiled_format_new(PyTypeObject *subtype_p,
-                                     PyObject *format_p)
+static PyObject *compiled_format_create(PyTypeObject *type_p,
+                                        PyObject *format_p)
 {
-    struct compiled_format_t *self_p;
+    PyObject *self_p;
 
-    self_p = (struct compiled_format_t *)subtype_p->tp_alloc(subtype_p, 0);
+    self_p = compiled_format_new(type_p, NULL, NULL);
 
-    if (self_p != NULL) {
-        self_p->info_p = parse_format(format_p);
-
-        if (self_p->info_p == NULL) {
-            PyObject_Free(self_p);
-            self_p = NULL;
-        }
+    if (self_p == NULL) {
+        return (NULL);
     }
 
-    return ((PyObject *)self_p);
+    if (compiled_format_init_inner((struct compiled_format_t *)self_p,
+                                   format_p) != 0) {
+        return (NULL);
+    }
+
+    return (self_p);
+}
+
+static PyObject *compiled_format_new(PyTypeObject *type_p,
+                                     PyObject *args_p,
+                                     PyObject *kwargs_p)
+{
+    return (type_p->tp_alloc(type_p, 0));
+}
+
+static int compiled_format_init(struct compiled_format_t *self_p,
+                                PyObject *args_p,
+                                PyObject *kwargs_p)
+{
+    int res;
+    PyObject *format_p;
+
+    res = PyArg_ParseTuple(args_p, "O", &format_p);
+
+    if (res == 0) {
+        return (-1);
+    }
+
+    return (compiled_format_init_inner(self_p, format_p));
+}
+
+static int compiled_format_init_inner(struct compiled_format_t *self_p,
+                                      PyObject *format_p)
+{
+    self_p->info_p = parse_format(format_p);
+
+    if (self_p->info_p == NULL) {
+        PyObject_Free(self_p);
+
+        return (-1);
+    }
+
+    Py_INCREF(format_p);
+    self_p->format_p = format_p;
+
+    return (0);
 }
 
 static void compiled_format_dealloc(struct compiled_format_t *self_p)
 {
     PyMem_RawFree(self_p->info_p);
+    Py_DECREF(self_p->format_p);
 }
 
 static PyObject *m_compiled_format_pack(struct compiled_format_t *self_p,
@@ -1633,82 +1899,176 @@ static PyObject *m_compiled_format_calcsize(struct compiled_format_t *self_p)
     return (calcsize(self_p->info_p));
 }
 
-static struct PyMethodDef compiled_format_methods[] = {
-    {
-        "pack",
-        (PyCFunction)m_compiled_format_pack,
-        METH_VARARGS,
-        pack___doc__
-    },
-    {
-        "unpack",
-        (PyCFunction)m_compiled_format_unpack,
-        METH_VARARGS,
-        unpack___doc__
-    },
-    {
-        "pack_into",
-        (PyCFunction)m_compiled_format_pack_into,
-        METH_VARARGS | METH_KEYWORDS,
-        pack_into___doc__
-    },
-    {
-        "unpack_from",
-        (PyCFunction)m_compiled_format_unpack_from,
-        METH_VARARGS | METH_KEYWORDS,
-        unpack_from___doc__
-    },
-    {
-        "calcsize",
-        (PyCFunction)m_compiled_format_calcsize,
-        METH_NOARGS,
-        calcsize___doc__
-    },
-    { NULL }
-};
-
-static PyTypeObject compiled_format_type = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    .tp_name = "bitstruct.c.CompiledFormat",
-    .tp_doc = NULL,
-    .tp_basicsize = sizeof(struct compiled_format_t),
-    .tp_itemsize = 0,
-    .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
-    .tp_dealloc = (destructor)compiled_format_dealloc,
-    .tp_methods = compiled_format_methods,
-};
-
-static PyObject *compiled_format_dict_new(PyTypeObject *subtype_p,
-                                          PyObject *format_p,
-                                          PyObject *names_p)
+static PyObject *m_compiled_format_copy(struct compiled_format_t *self_p)
 {
-    struct compiled_format_dict_t *self_p;
+    struct compiled_format_t *new_p;
+    size_t info_size;
 
-    if (!is_names_list(names_p)) {
+    new_p = (struct compiled_format_t *)compiled_format_new(
+        &compiled_format_type,
+        NULL,
+        NULL);
+
+    if (new_p == NULL) {
         return (NULL);
     }
 
-    self_p = (struct compiled_format_dict_t *)subtype_p->tp_alloc(subtype_p, 0);
+    info_size = sizeof(*self_p->info_p);
+    info_size += (sizeof(self_p->info_p->fields[0])
+                  * (self_p->info_p->number_of_fields - 1));
 
-    if (self_p != NULL) {
-        self_p->info_p = parse_format(format_p);
+    new_p->info_p = PyMem_RawMalloc(info_size);
 
-        if (self_p->info_p == NULL) {
-            PyObject_Free(self_p);
-            self_p = NULL;
-        } else {
-            Py_INCREF(names_p);
-            self_p->names_p = names_p;
-        }
+    if (new_p->info_p == NULL) {
+        /* ToDo: Free new_p. */
+        return (NULL);
     }
 
-    return ((PyObject *)self_p);
+    memcpy(new_p->info_p, self_p->info_p, info_size);
+    Py_INCREF(self_p->format_p);
+    new_p->format_p = self_p->format_p;
+
+    return ((PyObject *)new_p);
+}
+
+static PyObject *m_compiled_format_deepcopy(struct compiled_format_t *self_p,
+                                            PyObject *args_p)
+{
+    return (m_compiled_format_copy(self_p));
+}
+
+static PyObject *m_compiled_format_getstate(struct compiled_format_t *self_p,
+                                            PyObject *args_p)
+{
+    return (Py_BuildValue("{sOsi}",
+                          "format",
+                          self_p->format_p,
+                          pickle_version_key,
+                          pickle_version));
+}
+
+static PyObject *m_compiled_format_setstate(struct compiled_format_t *self_p,
+                                            PyObject *state_p)
+{
+    PyObject *version_p;
+    int version;
+    PyObject *format_p;
+
+    if (!PyDict_CheckExact(state_p)) {
+        PyErr_SetString(PyExc_ValueError, "Pickled object is not a dict.");
+
+        return (NULL);
+    }
+
+    version_p = PyDict_GetItemString(state_p, pickle_version_key);
+
+    if (version_p == NULL) {
+        PyErr_Format(PyExc_KeyError,
+                     "No \"%s\" in pickled dict.",
+                     pickle_version_key);
+
+        return (NULL);
+    }
+
+    version = (int)PyLong_AsLong(version_p);
+
+    if (version != pickle_version) {
+        PyErr_Format(PyExc_ValueError,
+                     "Pickle version mismatch. Got version %d but expected version %d.",
+                     version, pickle_version);
+
+        return (NULL);
+    }
+
+    format_p = PyDict_GetItemString(state_p, "format");
+
+    if (format_p == NULL) {
+        PyErr_SetString(PyExc_KeyError, "No \"format\" in pickled dict.");
+
+        return (NULL);
+    }
+
+    if (compiled_format_init_inner(self_p, format_p) != 0) {
+        return (NULL);
+    }
+
+    Py_RETURN_NONE;
+}
+
+static PyObject *compiled_format_dict_create(PyTypeObject *type_p,
+                                             PyObject *format_p,
+                                             PyObject *names_p)
+{
+    PyObject *self_p;
+
+    self_p = compiled_format_dict_new(type_p, NULL, NULL);
+
+    if (self_p == NULL) {
+        return (NULL);
+    }
+
+    if (compiled_format_dict_init_inner((struct compiled_format_dict_t *)self_p,
+                                        format_p,
+                                        names_p) != 0) {
+        return (NULL);
+    }
+
+    return (self_p);
+}
+
+static PyObject *compiled_format_dict_new(PyTypeObject *type_p,
+                                          PyObject *args_p,
+                                          PyObject *kwargs_p)
+{
+    return (type_p->tp_alloc(type_p, 0));
+}
+
+static int compiled_format_dict_init(struct compiled_format_dict_t *self_p,
+                                     PyObject *args_p,
+                                     PyObject *kwargs_p)
+{
+    int res;
+    PyObject *format_p;
+    PyObject *names_p;
+
+    res = PyArg_ParseTuple(args_p, "OO", &format_p, &names_p);
+
+    if (res == 0) {
+        return (-1);
+    }
+
+    return (compiled_format_dict_init_inner(self_p, format_p, names_p));
+}
+
+static int compiled_format_dict_init_inner(struct compiled_format_dict_t *self_p,
+                                           PyObject *format_p,
+                                           PyObject *names_p)
+{
+    if (!is_names_list(names_p)) {
+        return (-1);
+    }
+
+    self_p->info_p = parse_format(format_p);
+
+    if (self_p->info_p == NULL) {
+        PyObject_Free(self_p);
+
+        return (-1);
+    }
+
+    Py_INCREF(format_p);
+    self_p->format_p = format_p;
+    Py_INCREF(names_p);
+    self_p->names_p = names_p;
+
+    return (0);
 }
 
 static void compiled_format_dict_dealloc(struct compiled_format_dict_t *self_p)
 {
     PyMem_RawFree(self_p->info_p);
     Py_DECREF(self_p->names_p);
+    Py_DECREF(self_p->format_p);
 }
 
 static PyObject *m_compiled_format_dict_pack(struct compiled_format_dict_t *self_p,
@@ -1794,56 +2154,120 @@ static PyObject *m_compiled_format_dict_calcsize(
     return (calcsize(self_p->info_p));
 }
 
-static struct PyMethodDef compiled_format_dict_methods[] = {
-    {
-        "pack",
-        (PyCFunction)m_compiled_format_dict_pack,
-        METH_O,
-        pack___doc__
-    },
-    {
-        "unpack",
-        (PyCFunction)m_compiled_format_dict_unpack,
-        METH_O,
-        unpack___doc__
-    },
-    {
-        "pack_into",
-        (PyCFunction)m_compiled_format_dict_pack_into,
-        METH_VARARGS | METH_KEYWORDS,
-        pack_into___doc__
-    },
-    {
-        "unpack_from",
-        (PyCFunction)m_compiled_format_dict_unpack_from,
-        METH_VARARGS | METH_KEYWORDS,
-        unpack_from___doc__
-    },
-    {
-        "calcsize",
-        (PyCFunction)m_compiled_format_dict_calcsize,
-        METH_NOARGS,
-        calcsize___doc__
-    },
-    { NULL }
-};
+static PyObject *m_compiled_format_dict_copy(struct compiled_format_dict_t *self_p)
+{
+    struct compiled_format_dict_t *new_p;
+    size_t info_size;
 
-static PyTypeObject compiled_format_dict_type = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    .tp_name = "bitstruct.c.CompiledFormatDict",
-    .tp_doc = NULL,
-    .tp_basicsize = sizeof(struct compiled_format_dict_t),
-    .tp_itemsize = 0,
-    .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
-    .tp_dealloc = (destructor)compiled_format_dict_dealloc,
-    .tp_methods = compiled_format_dict_methods,
-};
+    new_p = (struct compiled_format_dict_t *)compiled_format_dict_new(
+        &compiled_format_dict_type,
+        NULL,
+        NULL);
+
+    if (new_p == NULL) {
+        return (NULL);
+    }
+
+    info_size = sizeof(*self_p->info_p);
+    info_size += (sizeof(self_p->info_p->fields[0])
+                  * (self_p->info_p->number_of_fields - 1));
+
+    new_p->info_p = PyMem_RawMalloc(info_size);
+
+    if (new_p->info_p == NULL) {
+        /* ToDo: Free new_p. */
+        return (NULL);
+    }
+
+    memcpy(new_p->info_p, self_p->info_p, info_size);
+    Py_INCREF(self_p->names_p);
+    new_p->names_p = self_p->names_p;
+    Py_INCREF(self_p->format_p);
+    new_p->format_p = self_p->format_p;
+
+    return ((PyObject *)new_p);
+}
+
+static PyObject *m_compiled_format_dict_deepcopy(struct compiled_format_dict_t *self_p,
+                                                 PyObject *args_p)
+{
+    return (m_compiled_format_dict_copy(self_p));
+}
 
 PyDoc_STRVAR(compile___doc__,
              "compile(fmt, names=None)\n"
              "--\n"
              "\n");
 
+static PyObject *m_compiled_format_dict_getstate(struct compiled_format_dict_t *self_p,
+                                                 PyObject *args_p)
+{
+    return (Py_BuildValue("{sOsOsi}",
+                          "format",
+                          self_p->format_p,
+                          "names",
+                          self_p->names_p,
+                          pickle_version_key,
+                          pickle_version));
+}
+
+static PyObject *m_compiled_format_dict_setstate(struct compiled_format_dict_t *self_p,
+                                                 PyObject *state_p)
+{
+    PyObject *version_p;
+    int version;
+    PyObject *format_p;
+    PyObject *names_p;
+
+    if (!PyDict_CheckExact(state_p)) {
+        PyErr_SetString(PyExc_ValueError, "Pickled object is not a dict.");
+
+        return (NULL);
+    }
+
+    version_p = PyDict_GetItemString(state_p, pickle_version_key);
+
+    if (version_p == NULL) {
+        PyErr_Format(PyExc_KeyError,
+                     "No \"%s\" in pickled dict.",
+                     pickle_version_key);
+
+        return (NULL);
+    }
+
+    version = (int)PyLong_AsLong(version_p);
+
+    if (version != pickle_version) {
+        PyErr_Format(PyExc_ValueError,
+                     "Pickle version mismatch. Got version %d but expected version %d.",
+                     version, pickle_version);
+
+        return (NULL);
+    }
+
+    format_p = PyDict_GetItemString(state_p, "format");
+
+    if (format_p == NULL) {
+        PyErr_SetString(PyExc_KeyError, "No \"format\" in pickled dict.");
+
+        return (NULL);
+    }
+
+    names_p = PyDict_GetItemString(state_p, "names");
+
+    if (names_p == NULL) {
+        PyErr_SetString(PyExc_KeyError, "No \"names\" in pickled dict.");
+
+        return (NULL);
+    }
+
+    if (compiled_format_dict_init_inner(self_p, format_p, names_p) != 0) {
+        return (NULL);
+    }
+
+    Py_RETURN_NONE;
+}
+
 static PyObject *m_compile(PyObject *module_p,
                            PyObject *args_p,
                            PyObject *kwargs_p)
@@ -1870,11 +2294,11 @@ static PyObject *m_compile(PyObject *module_p,
     }
 
     if (names_p == Py_None) {
-        return (compiled_format_new(&compiled_format_type, format_p));
+        return (compiled_format_create(&compiled_format_type, format_p));
     } else {
-        return (compiled_format_dict_new(&compiled_format_dict_type,
-                                         format_p,
-                                         names_p));
+        return (compiled_format_dict_create(&compiled_format_dict_type,
+                                            format_p,
+                                            names_p));
     }
 }
 
@@ -1960,13 +2384,6 @@ PyMODINIT_FUNC PyInit_c(void)
 {
     PyObject *module_p;
 
-    py_zero_p = PyLong_FromLong(0);
-    module_p = PyModule_Create(&module);
-
-    if (module_p == NULL) {
-        return (NULL);
-    }
-
     if (PyType_Ready(&compiled_format_type) < 0) {
         return (NULL);
     }
@@ -1975,5 +2392,32 @@ PyMODINIT_FUNC PyInit_c(void)
         return (NULL);
     }
 
+    py_zero_p = PyLong_FromLong(0);
+    module_p = PyModule_Create(&module);
+
+    if (module_p == NULL) {
+        return (NULL);
+    }
+
+    Py_INCREF(&compiled_format_type);
+
+    if (PyModule_AddObject(module_p,
+                           "CompiledFormat",
+                           (PyObject *)&compiled_format_type) < 0) {
+        Py_DECREF(&compiled_format_type);
+        Py_DECREF(module_p);
+
+        return (NULL);
+    }
+
+    if (PyModule_AddObject(module_p,
+                           "CompiledFormatDict",
+                           (PyObject *)&compiled_format_dict_type) < 0) {
+        Py_DECREF(&compiled_format_dict_type);
+        Py_DECREF(module_p);
+
+        return (NULL);
+    }
+
     return (module_p);
 }
